﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using EnvelopeCreator.Converters;
using EnvelopeCreatorProject.DocusignAPI.Authentication;
using EnvelopeCreatorProject.DocusignAPI.RecipientType;
using EnvelopeCreatorProject.Exceptions;
using Newtonsoft.Json;

namespace EnvelopeCreatorProject.DocusignAPI
{
    [JsonConverter(typeof(RecipientConverter))]
    public class Recipient
    {
        [DefaultValue("")]
        public string ID { get; set; }

        [DefaultValue("")]
        public string UserName { get; set; }

        [DefaultValue("")]
        public string Email { get; set; }

        public int RoutingOrder { get; set; }

        [DefaultValue("")]
        public string RoleName { get; set; }
        
        public BaseRecipientType RecipientType { get; set; }

        string _sendType;

        public string SendType
        {
            get
            {
                var type = RecipientType?.GetType().ToString() ?? null;

                if (!string.IsNullOrEmpty(type))
                    return type;

                if (!string.IsNullOrEmpty(_sendType))
                    return _sendType;

                return null;
            }
            set
            {
                _sendType = value;
            }
        }

        [DefaultValue("")]
        public string AccessCode { get; set; }

        public RecipientAuthentication RecipientAuthentication { get; set; }
        
        public RecipientSignatureInfo SignatureInfo { get; set; }

        public bool InheritEmailNotificationConfiguration { get; set; }

        public Recipient()
        {
            Email = string.Empty;
            UserName = string.Empty;
            ID = string.Empty;
            SignatureInfo = new RecipientSignatureInfo();
            AccessCode = string.Empty;
            RoleName = string.Empty;
            RecipientAuthentication = new RecipientAuthentication();
            //By default, no auth is required, but set it to the base class with no IDCheckName
        }

        internal RecipientTypeCode GetType()
        {
            if(RecipientType == null)
                throw new RecipientTypeNullException(this);

            return RecipientType.GetType();
        }

        public void BuildRecipient()
        {
            RecipientType = BuildRecipient(SendType);
        }

        BaseRecipientType BuildRecipient(string recipientType)
            => Enum.TryParse<RecipientTypeCode>(recipientType, out var typecode)
            && typesToRecipient.TryGetValue(typecode, out var recipient)
            ? recipient : throw new RecipientTypeNullException(null);

        Dictionary<RecipientTypeCode, BaseRecipientType> typesToRecipient = new Dictionary<RecipientTypeCode, BaseRecipientType>()
        {
            { RecipientTypeCode.Agent, new Agent() },
            { RecipientTypeCode.CarbonCopy,  new CarbonCopy() },
            { RecipientTypeCode.Signer, new Signer() },
            { RecipientTypeCode.InPersonSigner, new InPersonSigner() },
            { RecipientTypeCode.CertifiedDelivery, new CertifiedDelivery() },
            { RecipientTypeCode.Editor, new Editor() },
            { RecipientTypeCode.Intermediary, new Intermediary() }
        };
    }
}
