﻿using System.Linq;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using System.Text;

namespace EnvelopeCreatorProject.Models.Builders
{
    internal class EnvelopeSettingsBuilder : IEnvelopeBuilder
    {
        private readonly EnvelopeSettings _settings;

        internal EnvelopeSettingsBuilder(EnvelopeSettings settings) {
            _settings = settings;
        }

        public void BuildTo(StringBuilder strBuilder)
        { 
            strBuilder.AppendLine("\"allowMarkup\":\"" +_settings.AllowMarkup.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"allowReassign\":\"" + _settings.AllowReassign.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"allowRecipientRecursion\":\"" + _settings.AllowRecipientRecursion.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"asynchronous\":\"" + _settings.Asynchronous.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"authoritativeCopy\":\"" + _settings.AuthoritativeCopy.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"autoNavigation\":\"" + _settings.AutoNavigation.ToString().ToLower() + "\",");
            if (!string.IsNullOrEmpty(_settings.BrandId)) strBuilder.AppendLine("\"brandId\":\"" + _settings.BrandId + "\",");
            strBuilder.AppendLine("\"brandLock\":\"" + _settings.BrandLock.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"enableWetSign\":\"" + _settings.EnableWetSign.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"messageLock\":\"" + _settings.MessageLock.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"recipientsLock\":\"" + _settings.RecipientsLock.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"signingLocation\":\"" + GetSigningLocation(_settings.SigningLocation) + "\",");
            if (!string.IsNullOrEmpty(_settings.TransactionId)) strBuilder.AppendLine("\"transactionId\":\"" + _settings.TransactionId + "\",");
            strBuilder.AppendLine("\"useDisclosure\":\"" + _settings.UseDisclosure.ToString().ToLower() + "\",");
            strBuilder.AppendLine("\"enforceSignerVisibility\":\"" + _settings.EnforceSignerVisibility.ToString().ToLower() + "\",");
            if(_settings.Notification != null) strBuilder.AppendLine("\"notification\":" + GetNotification(_settings.Notification) + ",");
            if (_settings.EmailSettings != null) strBuilder.AppendLine("\"emailSettings\":" + GetEmailSettings(_settings.EmailSettings) + ",");
            if (_settings.CustomFields!= null) strBuilder.AppendLine("\"customFields\":" + GetCustomFields(_settings.CustomFields) + ",");
        }

        private string GetNotification(DocusignNotificationSettings docusignNotificationSettings)
        {
            return JsonConvert.SerializeObject(docusignNotificationSettings, new JsonSerializerSettings(){ContractResolver = new CamelCasePropertyNamesContractResolver()});
        }

        private string GetEmailSettings(DocusignEmailSettings emailSettings)
        {
            return
                string.Format(
                    "{{\"replyEmailAddressOverride\":\"{0}\",\"replyEmailNameOverride\":\"{1}\",\"bccEmailAddresses\":[{2}]}}",
                    emailSettings.ReplyEmailAddressOverride,
                    emailSettings.ReplyEmailNameOverride,
                    string.Join(",",
                        emailSettings.BccEmailAddresses.Select(x => string.Format("{{\"email\":\"{0}\"}}", x))));
        }

        private string GetCustomFields(DocusignCustomFields customFields)
        {
            return JsonConvert.SerializeObject(customFields, new JsonSerializerSettings() { ContractResolver = new CamelCasePropertyNamesContractResolver() });
        }

        private string GetSigningLocation(SignLocation signLocation)
        {
            return signLocation == SignLocation.Online ? "Online" : "InPerson";
        }
        
        

    }
}
